@@ -1,19 +1,12 @@
 context("filenames")
 
-source("utils.R")
 
 test_succeeds("filenames can be listed", {
 
-  dataset <- file_list_dataset("data/mtcars*.csv") %>%
+  dataset <-
+    testing_data_filepath("mtcars*.csv") %>%
+    file_list_dataset() %>%
     dataset_interleave(cycle_length = 2, function(file) {
       text_line_dataset(file)
     })
 })
-
-
-
-
-
-
-
-
